/**
 * An abstract class to save a number of moves. The count of moves is specified
 * in HISTORY_LENGTH. Each move is characterised through a direction (left
 * turn, right turn or forward) and a duration (e.g. 250msec). The moves were
 * saved in two cyclic arrays.
 * 
 * Objective of the class is to make the roboter be able to follow the line in
 * spite of interruption of line in bends. E.g. at a left turn the roboter will
 * drive forward and miss the line. This class calculates a move that avoids
 * missing (through a little left turn).  
 * 
 * @author tthuem
 */
public abstract class MoveHistory {
	
	/**
	 * the length of saved moves
	 */
	private final static byte HISTORY_LENGTH = 100;
	
	/**
	 * the length of saved moves decremented by one
	 */
	private final static byte MAX_INDEX = HISTORY_LENGTH - 1;
	
	/**
	 * the duration that was used to calculate the move
	 */
	private final static short MSEC_FOR_PREDICTION = 500;
	
	/**
	 * an array containing the directions of the moves
	 */
	private static byte[] dir = new byte[HISTORY_LENGTH];
	
	/**
	 * an array containing the durations of the moves
	 */
	private static short[] dur = new short[HISTORY_LENGTH];
	
	/**
	 * an index pointing at the last move
	 */
	private static byte index = 0;
	
	static {
		dur[index] = 10000;
	}
	
	/**
	 * Saves a move in the cyclic arrays and increments the index.
	 * 
	 * @param direction
	 * @param duration
	 */
	public static void add(byte direction, short duration) {
		if (direction < -1 || direction > 1) return;
		index = (byte)(++index % HISTORY_LENGTH);
		dir[index] = direction;
		dur[index] = duration;
	}
	
	/**
	 * Calculates and returns the duration in msec to turn left or right. The
	 * direction was represented by the sign of the returned short. 
	 * 
	 * @return
	 */
	public static short getPredictedMove() {
		short sum = 0;
		short sumL = 0;
		short sumR = 0;
		byte i = index;
		while (sum + dur[i] < MSEC_FOR_PREDICTION) {
			
			sum += dur[i]; 
			sumL += dir[i] * dur[i]; 
			sumR -= dir[i] * dur[i];
			i = (byte)((i + MAX_INDEX) % HISTORY_LENGTH);
		}
		short remainder = (short) (MSEC_FOR_PREDICTION - sum);
		sumL += dir[i] * remainder; 
		sumR -= dir[i] * remainder; 
		return (short) ((sumL - sumR) / 4);
	}

//	/**
//	 * An example to test the class.
//	 * 
//	 * @param args
//	 */
//	public static void main(String[] args) {
//		add((byte)-1, (short)250);
//		add((byte)+1, (short)350);
//		add((byte) 0, (short)500);
//		System.out.println(getPredictedMove());
//	}

}
